import axios from 'axios';
import ExceptionHandler from '../Exceptions/ExceptionHandler.js';
import CardapioInteligente from './CardapioInteligente.js';
import Optimizer from './Optimizer.js';
import fs from 'fs';
const Algoritmo = {

    async call( params ){
        const { url, body } = params;
        let paramsAxios = {
            method: 'POST',
            url,
            headers: {'Content-Type': 'application/json'},
            data: body
        };
        const result = await axios(paramsAxios);
        return result;
    },
    
    async checkGeneration( dataSet ){
        const validations = {
            CUSTO_FORA_META: data => {
                const {
                    custo_pdr,
                    pu_aculm,
                    dif_custo
                } = data;
                if(dif_custo > 0) return [];
                const toFixed = value => value.toFixed(5);
                const message = `
                    Custo Padrão: ${toFixed(custo_pdr)} 
                    - P.U. Gerado: ${toFixed(pu_aculm)} 
                    - Diferença: ${toFixed(dif_custo)}
                `.replace(/\n|\s+/gi, ' ').trim();
                return [ message ];
            },
            INCIDENCIA: data => {
                const {
                    frequencias,
                    freqData
                } = data;
                let messages = [];
                if(!frequencias || !frequencias.length) return [];
                Object.values(freqData).forEach( agrupamento => {
                    let { GRUPOS } = agrupamento;
                    GRUPOS = Object.values(GRUPOS);
                    if( GRUPOS ){
                        GRUPOS.forEach( grupo => {
                            const { NMEXIB: NOME_GRUPO_PRATO } = grupo;
                            let [MIN_MES_GRP, MAX_MES_GRP, MAX_SEM_GRP, TOTAL_MES, _, USO_POR_SEMANA] = grupo.freq;
                            if(grupo.freqError){
                                messages.push(`
                                Grupo de Prato: ${NOME_GRUPO_PRATO} 
                                - Mín. Mês: ${MIN_MES_GRP} 
                                - Máx. Mês: ${MAX_MES_GRP} - Total: ${TOTAL_MES}
                                `.replace(/\n|\s+/gi, ' ').trim());
                            }
                            if(grupo.freqErrorWeek){
                                Object.keys( USO_POR_SEMANA ).forEach( weekNumber => {
                                    let valueInWeek = USO_POR_SEMANA[weekNumber];
                                    if(valueInWeek > MAX_SEM_GRP){
                                        messages.push(`
                                        Grupo de Prato: ${NOME_GRUPO_PRATO}
                                        - Máx. Semana: ${MAX_SEM_GRP}
                                        - Total: ${valueInWeek}
                                        `.replace(/\n|\s+/gi, ' ').trim());
                                    }
                                })
                            }
                        });
                    }
                })
                return messages;
            },
            BURACO: data => {
                let result = {};
                let messages = [];
                if(!data || !data.aditionalLogInfo) return [];
                const { diasComBuracos } = data.aditionalLogInfo;
                let [ dishTypes ] = data.tabela.pratos;
                let mappedDishTypes = dishTypes.reduce( (acc, cv) => {
                    acc[cv.seq] = cv.name;
                    return acc;
                }, {});
                diasComBuracos.forEach( diaBuraco => {
                    let pratos = diaBuraco.pratos.filter( item => !item.prato && item.otherOptions.length > 0);
                    pratos.forEach( prato => {
                        messages.push(`
                            Data: ${diaBuraco.dia}
                            - Tipo de Prato: ${mappedDishTypes[prato.sequencial]}
                        `.replace(/\n|\s+/gi, ' ').trim());
                    });
                })
                return messages;
            }
        };
        const {
            NRITGERACAOCARD,
            NRORG,
            CDOPERADOR
        } = dataSet;
        let logParameters = {
            NRITGERACAOCARD,
            NRORG,
            CDOPERADOR,
            DSLOGINTERNO: null
        };
        let promises = [];
        Object.keys(validations).forEach( validation => {
            try {
                let result = validations[validation].apply(this, [dataSet]);
                logParameters.IDTIPOLOG = validation;
                if(result.length){
                    result.forEach( DSLOGGERACAOCARD => {
                        logParameters.DSLOGGERACAOCARD = DSLOGGERACAOCARD;
                        let parametersCopy = Object.assign({}, logParameters); // fix error object reference change in loop + async insert database
                        promises.push(CardapioInteligente.insereLog(parametersCopy));
                    })
                }
            } catch (error) {
                console.log(error);
            }
        });
        return Promise.all(promises);
    }

}

export default Algoritmo;
