import CardapioInteligente from './CardapioInteligente.js';
import Algoritmo from './Algoritmo.js';
import Optimizer from './Optimizer.js';
import Status from './Status.js';

const OtimizadorLote = {
    async testConnection(){
        console.log(process.env.HOST_ALGORITMO_SERVER);
        const result = await CardapioInteligente.testConnection();
        return result;
    },
    async getParametersGenData( params ){
        const { NRGERACAOCARD, NRITGERACAOCARD, CDOPERADOR, NRORG } = params;
        const backOfficeData = await CardapioInteligente.getBackOfficeData(NRGERACAOCARD, NRITGERACAOCARD);
        let paramsGetGenData = {
            IDTIPORESTRICAO: backOfficeData.IDTIPORESTRICAO,
            DTREFCUSPADR   : backOfficeData.DTMESCARD,
            DTMES          : backOfficeData.DTMESCARD,
            DIAS_CONG      : backOfficeData.QTDIASCONG,
            VLR_CONGELADO  : backOfficeData.QTDIASCONG,
            CDNVPRATO      : backOfficeData.CDPICTPRAT.match(/\./g).length+1,
            CDOPERADOR,
            NRORG
        };
        return Object.assign({}, params, paramsGetGenData, backOfficeData);
    },
    async isValidBatchItem( NRGERACAOCARD, NRITGERACAOCARD, NRORG, ROW_ID, socket ){
        const IS_VALID = await CardapioInteligente.isValidServiceLink(NRGERACAOCARD, NRITGERACAOCARD);
        if(!IS_VALID){
            const deleteResponse = await CardapioInteligente.deleteOldLog( NRITGERACAOCARD );
            const ERROR = 'Serviço possui tipos de pratos vinculados mas não possui cenário vinculado informado.';
            const result = await CardapioInteligente.insereLog({
                NRITGERACAOCARD,
                IDTIPOLOG: 'UNEXPECTED_ERROR',
                DSLOGGERACAOCARD: ERROR,
                NRORG
            });
            const response = {
                ROW_ID,
                STATUS: 'ERRO',
                data: ERROR,
                OPERATION: "GENERATE",
                NRITGERACAOCARD
            };
            socket.emit('onSocketSceneryBatchMessage', response);
        }
        return IS_VALID;
    },
    async otimizar( requestParams, socket ){
       const timeStart = new Date().getTime();
        const { CDOPERADOR, NRORG, NRGERACAOCARD, NRITGERACAOCARD, ROW_ID } = requestParams;
        const IS_VALID_BATCH_ITEM = await this.isValidBatchItem(NRGERACAOCARD, NRITGERACAOCARD, NRORG, ROW_ID, socket);
        if( !IS_VALID_BATCH_ITEM ) return null;
        const params = await this.getParametersGenData( requestParams );
        const dataSet = await CardapioInteligente.retDadosCardInteligente( params );
        dataSet.parsedCardapio = params.parsedCardapio;
        CardapioInteligente.addIncidencia_tp_gp_prato(dataSet);
        const { VRCUSTPADR, VLR_CUSTO_MAX, VLR_CUSTO_MIN } = params;
        Object.assign(dataSet, {
            VRCUSTPADR, VLR_CUSTO_MAX, VLR_CUSTO_MIN, CDOPERADOR, NRORG, 
            P_DTMES      : params.DTMESCARD,
            P_VRCUSTPADR : VRCUSTPADR
        });
        console.log(`Call algoritmo from server: ${process.env.HOST_ALGORITMO_SERVER}`);
        const paramsAlgoritmo = {
            url : process.env.HOST_ALGORITMO_SERVER,
            body: dataSet 
        };
        console.log('Finish genData...');
        let error = null;
        const optimized = await Algoritmo.call(paramsAlgoritmo).catch( err => {
            if(err.response && err.response.data){
                console.log(err.response.data);
                error = err.response.data;
                if(error.title === "ERRO_EXECUCAO"){
                    error.DSLOGINTERNO = err.response.data.otmmessage;
                    error.otmmessage = "Ocorreu um erro inesperado na execução do algoritmo";
                }
                error.title = "UNEXPECTED_ERROR";
            } else {
                console.log(err);
                error = {
                    success: false,
                    otmmessage: `Erro ao chamar o algoritimo de otimização ${paramsAlgoritmo.url}`,
                    title: "UNEXPECTED_ERROR"
                }
            }
        });
        let response = null;
        let promises = [];
        console.log('Finish call Algoritmo...');
        console.log(error);
        const deleteResponse = await CardapioInteligente.deleteOldLog( params.NRITGERACAOCARD );
        if( error ){
            error.IDTIPOLOG = error.title;
            error.DSLOGGERACAOCARD = error.otmmessage;
            response = {
                STATUS: 'ERRO',
                data: error,
                OPERATION: "GENERATE"
            };
            Object.assign(response, requestParams);
            let logParameters = Object.assign({}, requestParams, error);
            const result = await CardapioInteligente.insereLog(logParameters);
            const responseStatus = await Status.updateStatusItGeracaoCard(params.NRITGERACAOCARD, "ERRO");
            console.log('Finish insert/update logs status');

            socket.emit('onSocketSceneryBatchMessage', response);
        } else {
            const paramCenario = optimized.data.data;
            paramCenario.NRCENARIOVINCULO = params.NRCENARIOVINCULO;
            paramCenario.backOfficeData = params;
            Object.assign(paramCenario,  params);
            paramCenario.freqData = Optimizer.feedFreqData( paramCenario );
            paramCenario.JSONCENARIO = JSON.stringify(paramCenario);
            paramCenario.JSONCARDAPIO = '{}';
            paramCenario.incidencia = dataSet.incidencia;
            console.log(paramCenario.pu_aculm);
            paramCenario.pu_aculm = paramCenario.pu_aculm 
                ? parseFloat(parseFloat(paramCenario.pu_aculm).toFixed(5))
                : 0;
            paramCenario.INGData = Optimizer.feedINGData( paramCenario );
            promises.push(Algoritmo.checkGeneration(paramCenario));
            promises.push(CardapioInteligente.insereCenario(paramCenario));
            Promise.all(promises).then( async resp => {
                const [ LOGITGERACAOCARD, CENARIO ] = resp;
                const STATUS = LOGITGERACAOCARD.length ? "FINALIZADO_COM_ERRO" : "FINALIZADO_SEM_ERRO";
                const responseSocket = {
                    OPERATION: "GENERATE",
                    NRITGERACAOCARD: params.NRITGERACAOCARD, 
                    ROW_ID: params.ROW_ID, 
                    STATUS, 
                    NRCENARIOCARD: parseInt(CENARIO.NRCENARIOCARD), 
                    TIME: new Date().getTime() - timeStart
                };
                const responseStatus = await Status.updateStatusItGeracaoCard(params.NRITGERACAOCARD, STATUS);
                socket.emit('onSocketSceneryBatchMessage', responseSocket);
            }).catch( error => {
                socket.emit('onSocketSceneryBatchMessage', {
                    OPERATION: "GENERATE",
                    ROW_ID: params.ROW_ID,
                    STATUS: "ERRO",
                    NRITGERACAOCARD: params.NRITGERACAOCARD,
                    message: error.message, stack: error.stack
                });
            });
        }
    }
};

export default OtimizadorLote;